%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
%   mosaic funtion
% 
%    Input
%     - im : either full RGB image or mosaiced 1-channel image
%     - pattern : mosaic pattern
%           pattern = 'grbg'
%            G R ..
%            B G ..
%            : : 
%           pattern = 'rggb'
%            R G ..
%            G B ..
%            : :    
%           pattern = 'gbrg'
%            G B ..
%            R G ..
%            : : 
%           pattern = 'bggr'
%            B G ..
%            G R ..
%            : :    
% 
%    Output
%     - mosaic :  mosaiced image
%     - mask   :  binaly mask (3D data : height*width*RGB)
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [mosaic mask] = mosaic_bayer(im, pattern)

num = zeros(size(pattern));
num(find((pattern == 'r') + (pattern == 'R'))) = 1;
num(find((pattern == 'g') + (pattern == 'G'))) = 2;
num(find((pattern == 'b') + (pattern == 'B'))) = 3;

H = size(im,1);
W = size(im,2);

if size(im,3) == 4,
	H = 2*H;
	W = 2*W;
end

mosaic = zeros(H, W, 3);
mask   = zeros(H, W, 3);

rows1 = 1:2:H;
rows2 = 2:2:H;
cols1 = 1:2:W;
cols2 = 2:2:W;

mask(rows1, cols1, num(1)) = 1;
mask(rows1, cols2, num(2)) = 1;
mask(rows2, cols1, num(3)) = 1;
mask(rows2, cols2, num(4)) = 1;

if size(im,3) == 3,
	mosaic(:,:,1) = im(:,:,1) .* mask(:,:,1);
	mosaic(:,:,2) = im(:,:,2) .* mask(:,:,2);
	mosaic(:,:,3) = im(:,:,3) .* mask(:,:,3);
elseif size(im,3) == 1,
	mosaic(rows1, cols1, num(1)) = im(rows1, cols1);
	mosaic(rows1, cols2, num(2)) = im(rows1, cols2);
	mosaic(rows2, cols1, num(3)) = im(rows2, cols1);
	mosaic(rows2, cols2, num(4)) = im(rows2, cols2);
else
	num4 = zeros(size(pattern));
	num4(find((pattern == 'r') + (pattern == 'R'), 1, 'first')) = 2;
	num4(find((pattern == 'g') + (pattern == 'G'), 1, 'first')) = 1;
	num4(find((pattern == 'g') + (pattern == 'G'), 1, 'last' )) = 4;
	num4(find((pattern == 'b') + (pattern == 'B'), 1, 'first')) = 3;

	mosaic(rows1, cols1, num(1)) = im(:, :, num4(1));
	mosaic(rows1, cols2, num(2)) = im(:, :, num4(2));
	mosaic(rows2, cols1, num(3)) = im(:, :, num4(3));
	mosaic(rows2, cols2, num(4)) = im(:, :, num4(4));
end
