from .base_options import BaseOptions


class TestOptions(BaseOptions):
    """This class includes test options.

    It also includes shared options defined in BaseOptions.
    """

    def initialize(self, parser):
        parser = BaseOptions.initialize(self, parser)  # define shared options
        parser.add_argument('--ntest', type=int, default=float("inf"), help='# of test examples.')
        parser.add_argument('--results_dir', type=str, default='./results/', help='saves results here.')
        parser.add_argument('--aspect_ratio', type=float, default=1.0, help='aspect ratio of result images')
        parser.add_argument('--phase', type=str, default='test', help='train, val, test, etc')
        # Dropout and Batchnorm have different behaviour during training and test.
        parser.add_argument('--first_imgs',  type=str, default='noisy', help='[ noisy, gt, denoised_by_noisyinputs] States which images to feed when starting a video in test_recurrency.py')
        parser.add_argument('--num_test', type=float, default=float("inf"), help='how many test images to run')
        parser.add_argument('--warp_from_flow',  type=str, default='denoised', help='[ denoised, n, gt] Warp following the optical flow computed between "warp_from_flow" frame and the current noisy frame.')
        parser.add_argument('--apply_pipe', action='store_true', default=False, help='Applies postp pipe to all visuals.')
        # # rewrite devalue values
        # parser.set_defaults(model='test')
        # To avoid cropping, the load_size should be the same as patch_width
        parser.set_defaults(load_size=parser.get_default('patch_width'))
        self.isTrain = False

        parser.set_defaults(dataset_mode='inferaxel')
        return parser
